from dataclasses import dataclass, field
from typing import List, Tuple
from netCDF4 import Dataset, num2date, chartostring

from functools import wraps

from mapies.mapies import MAPIES
from mapies.config.viirs_config import VIIRSData
from mapies.libs.func_tools import timeit, get_file_list, time_converter, time_domain_selection
import time
import numpy as np
import xarray as xr


class VIIRS(MAPIES):
    """
    Class for VIIRS specific data
    """
    
    def __init__(self, **kwargs):
        """
        Inherited init class with new variables
        """
        super().__init__()
        self.info = VIIRSData(**kwargs)
        self.input_file = "/esarchive/scratch/cmeikle/original-data-downloads/viirs/original_files/laads/AERDB_L2_VIIRS_NOAA20.A2024196.1848.002.nrt.nc"
        
        self.renaming_dict = {"Scan_Start_Time":"time", "Latitude":"lat", "Longitude":"lon"}
        self.start_date = "2024081604"
        self.end_date = "2024081605"
        
        if isinstance(self.start_date, str):
            self.start_date = time_converter(self.start_date)

        if isinstance(self.end_date, str):
            self.end_date = time_converter(self.end_date)

        
    def read_nc(self):
        """
        Inherited read netcdf class
        """
        super().read_nc()
        
        if self.name_changes:
            self.change_variable_names()
            
        self.time = np.ma.array(self.f["time"], fill_value = -9999)
        self.lon = np.ma.array(self.f["lon"], fill_value = -9999)
        self.lat = np.ma.array(self.f["lat"], fill_value = -9999)
        self.f.close()
        # Get a way of conserving the

    @timeit
    def read_nc_xarray(self):
        """
        Read netcdf files with xarray
        """

        files = get_file_list("/esarchive/obs/nasa/viirs_noaa20_aerdb_l2_nrt/original_files/VIIRS/2024/228/AERDB_L2_VIIRS_NOAA20.A2024228*")


        # Open dataset with xarray and dask
        ds = xr.open_mfdataset(files)

        # TODO Make these init values
        time_column = "Scan_Start_Time"
        

        # Time domain selection
        ds = time_domain_selection(ds, time_column, self.start_date, self.end_date)
        print(ds)

        # QA checks

        
        
    @property
    def dims(self):
        """
        Read the dimensions of the Satellite file
        """
        # Check if there are any new dims we were not expecting in the file
        return self.info.dims
    
    @property
    def coords(self):
        """
        Read the coodrs of the Satellite file
        """
        return self.info.coords
    
    @property
    def variables(self):
        """
        Read the data variables contained in the file
        """
        return self.info.variables

    @property
    def shape(self):
        """
        Shape of data passed
        """
        pass
        
    @property
    def name_changes(self):
        """
        Read the dimensions of the Satellite file
        """
        # Check if there are any new dims we were not expecting in the file
        return self.info.name_changes

        
    def AOD550(self):
        """Convert lats to masked array"""
        try:
            self.aod550 = np.ma.array(self.f["Aerosol_Optical_Thickness_550_Ocean_Best_Estimate"], fill_value = -9999)
        except Exception as e:
            print(f"No such variable, error {e}")
        
    def to_xarray(self):
        """
        Converts the Array like structure that we will have to a xarray dataset if someone wants to work with MAPIES data in that way
        """
        print(np.hstack(self.time))
        self.xrds = xr.Dataset({},
        coords = {'time' : self.time,
                    'lat' : self.lat,
                    'lon': self.lon})
        self.xrds["AOD550"] = self.aod550
        return self.xrds
    
if __name__ == "__main__":
    c = VIIRS()
    c.read_nc_xarray()